@@ -27,6 +27,7 @@ import six
 import swift.common.db
 from swift.account.backend import AccountBroker, DATADIR
 from swift.common.constraints import check_drive
+from swift.common.daemon import run_daemon
 from swift.common.direct_client import direct_delete_container, \
     direct_delete_object, direct_get_container
 from swift.common.exceptions import ClientException
@@ -34,7 +35,7 @@ from swift.common.request_helpers import USE_REPLICATION_NETWORK_HEADER
 from swift.common.ring import Ring
 from swift.common.ring.utils import is_local_device
 from swift.common.utils import get_logger, whataremyips, config_true_value, \
-    Timestamp, md5
+    Timestamp, md5, node_to_string, parse_options
 from swift.common.daemon import Daemon
 from swift.common.storage_policy import POLICIES, PolicyError
 
@@ -379,15 +380,14 @@ class AccountReaper(Daemon):
             except ClientException as err:
                 if self.logger.getEffectiveLevel() <= DEBUG:
                     self.logger.exception(
-                        'Exception with %(ip)s:%(port)s/%(device)s', node)
+                        'Exception with %s', node_to_string(node))
                 self.stats_return_codes[err.http_status // 100] = \
                     self.stats_return_codes.get(err.http_status // 100, 0) + 1
                 self.logger.increment(
                     'return_codes.%d' % (err.http_status // 100,))
             except (Timeout, socket.error):
                 self.logger.error(
-                    'Timeout Exception with %(ip)s:%(port)s/%(device)s',
-                    node)
+                    'Timeout Exception with %s', node_to_string(node))
             if not objects:
                 break
             try:
@@ -429,7 +429,7 @@ class AccountReaper(Daemon):
             except ClientException as err:
                 if self.logger.getEffectiveLevel() <= DEBUG:
                     self.logger.exception(
-                        'Exception with %(ip)s:%(port)s/%(device)s', node)
+                        'Exception with %s', node_to_string(node))
                 failures += 1
                 self.logger.increment('containers_failures')
                 self.stats_return_codes[err.http_status // 100] = \
@@ -438,8 +438,7 @@ class AccountReaper(Daemon):
                     'return_codes.%d' % (err.http_status // 100,))
             except (Timeout, socket.error):
                 self.logger.error(
-                    'Timeout Exception with %(ip)s:%(port)s/%(device)s',
-                    node)
+                    'Timeout Exception with %s', node_to_string(node))
                 failures += 1
                 self.logger.increment('containers_failures')
         if successes > failures:
@@ -506,7 +505,7 @@ class AccountReaper(Daemon):
             except ClientException as err:
                 if self.logger.getEffectiveLevel() <= DEBUG:
                     self.logger.exception(
-                        'Exception with %(ip)s:%(port)s/%(device)s', node)
+                        'Exception with %s', node_to_string(node))
                 failures += 1
                 self.logger.increment('objects_failures')
                 self.stats_return_codes[err.http_status // 100] = \
@@ -517,8 +516,7 @@ class AccountReaper(Daemon):
                 failures += 1
                 self.logger.increment('objects_failures')
                 self.logger.error(
-                    'Timeout Exception with %(ip)s:%(port)s/%(device)s',
-                    node)
+                    'Timeout Exception with %s', node_to_string(node))
             if successes > failures:
                 self.stats_objects_deleted += 1
                 self.logger.increment('objects_deleted')
@@ -528,3 +526,12 @@ class AccountReaper(Daemon):
             else:
                 self.stats_objects_possibly_remaining += 1
                 self.logger.increment('objects_possibly_remaining')
+
+
+def main():
+    conf_file, options = parse_options(once=True)
+    run_daemon(AccountReaper, conf_file, **options)
+
+
+if __name__ == '__main__':
+    main()
