@@ -28,19 +28,21 @@ from eventlet import Timeout
 
 from swift import __canonical_version__ as swift_version
 from swift.common import constraints
-from swift.common.http import is_server_error
+from swift.common.http import is_server_error, HTTP_INSUFFICIENT_STORAGE
 from swift.common.storage_policy import POLICIES
 from swift.common.ring import Ring
+from swift.common.error_limiter import ErrorLimiter
 from swift.common.utils import Watchdog, get_logger, \
     get_remote_client, split_path, config_true_value, generate_trans_id, \
     affinity_key_function, affinity_locality_predicate, list_from_csv, \
-    parse_prefixed_conf, config_auto_int_value, \
-    config_request_node_count_value, config_percent_value
+    parse_prefixed_conf, config_auto_int_value, node_to_string, \
+    config_request_node_count_value, config_percent_value, cap_length, \
+    parse_options
 from swift.common.registry import register_swift_info
 from swift.common.constraints import check_utf8, valid_api_version
 from swift.proxy.controllers import AccountController, ContainerController, \
     ObjectControllerRouter, InfoController
-from swift.proxy.controllers.base import get_container_info, NodeIter, \
+from swift.proxy.controllers.base import get_container_info, \
     DEFAULT_RECHECK_CONTAINER_EXISTENCE, DEFAULT_RECHECK_ACCOUNT_EXISTENCE, \
     DEFAULT_RECHECK_UPDATING_SHARD_RANGES, DEFAULT_RECHECK_LISTING_SHARD_RANGES
 from swift.common.swob import HTTPBadRequest, HTTPForbidden, \
@@ -48,6 +50,7 @@ from swift.common.swob import HTTPBadRequest, HTTPForbidden, \
     HTTPServerError, HTTPException, Request, HTTPServiceUnavailable, \
     wsgi_to_str
 from swift.common.exceptions import APIVersionError
+from swift.common.wsgi import run_wsgi
 
 
 # List of entry points for mandatory middlewares.
@@ -193,6 +196,10 @@ class Application(object):
 
     def __init__(self, conf, logger=None, account_ring=None,
                  container_ring=None):
+        # This is for the sake of tests which instantiate an Application
+        # directly rather than via loadapp().
+        self._pipeline_final_app = self
+
         if conf is None:
             conf = {}
         if logger is None:
@@ -200,7 +207,6 @@ class Application(object):
                                      statsd_tail_prefix='proxy-server')
         else:
             self.logger = logger
-        self._error_limiting = {}
         self.backend_user_agent = 'proxy-server %s' % os.getpid()
 
         swift_dir = conf.get('swift_dir', '/etc/swift')
@@ -214,10 +220,12 @@ class Application(object):
         self.client_chunk_size = int(conf.get('client_chunk_size', 65536))
         self.trans_id_suffix = conf.get('trans_id_suffix', '')
         self.post_quorum_timeout = float(conf.get('post_quorum_timeout', 0.5))
-        self.error_suppression_interval = \
+        error_suppression_interval = \
             float(conf.get('error_suppression_interval', 60))
-        self.error_suppression_limit = \
+        error_suppression_limit = \
             int(conf.get('error_suppression_limit', 10))
+        self.error_limiter = ErrorLimiter(error_suppression_interval,
+                                          error_suppression_limit)
         self.recheck_container_existence = \
             int(conf.get('recheck_container_existence',
                          DEFAULT_RECHECK_CONTAINER_EXISTENCE))
@@ -230,12 +238,16 @@ class Application(object):
         self.recheck_account_existence = \
             int(conf.get('recheck_account_existence',
                          DEFAULT_RECHECK_ACCOUNT_EXISTENCE))
+        self.container_existence_skip_cache = config_percent_value(
+            conf.get('container_existence_skip_cache_pct', 0))
         self.container_updating_shard_ranges_skip_cache = \
             config_percent_value(conf.get(
                 'container_updating_shard_ranges_skip_cache_pct', 0))
         self.container_listing_shard_ranges_skip_cache = \
             config_percent_value(conf.get(
                 'container_listing_shard_ranges_skip_cache_pct', 0))
+        self.account_existence_skip_cache = config_percent_value(
+            conf.get('account_existence_skip_cache_pct', 0))
         self.allow_account_management = \
             config_true_value(conf.get('allow_account_management', 'no'))
         self.container_ring = container_ring or Ring(swift_dir,
@@ -250,18 +262,8 @@ class Application(object):
                        [os.path.join(swift_dir, 'mime.types')])
         self.account_autocreate = \
             config_true_value(conf.get('account_autocreate', 'no'))
-        if conf.get('auto_create_account_prefix'):
-            self.logger.warning('Option auto_create_account_prefix is '
-                                'deprecated. Configure '
-                                'auto_create_account_prefix under the '
-                                'swift-constraints section of '
-                                'swift.conf. This option will '
-                                'be ignored in a future release.')
-            self.auto_create_account_prefix = \
-                conf['auto_create_account_prefix']
-        else:
-            self.auto_create_account_prefix = \
-                constraints.AUTO_CREATE_ACCOUNT_PREFIX
+        self.auto_create_account_prefix = \
+            constraints.AUTO_CREATE_ACCOUNT_PREFIX
         self.expiring_objects_account = self.auto_create_account_prefix + \
             (conf.get('expiring_objects_account_name') or 'expiring_objects')
         self.expiring_objects_container_divisor = \
@@ -286,6 +288,8 @@ class Application(object):
             if a.strip()]
         self.strict_cors_mode = config_true_value(
             conf.get('strict_cors_mode', 't'))
+        self.allow_open_expired = config_true_value(
+            conf.get('allow_open_expired', 'f'))
         self.node_timings = {}
         self.timing_expiry = int(conf.get('timing_expiry', 300))
         value = conf.get('request_node_count', '2 * replicas')
@@ -347,6 +351,7 @@ class Application(object):
             policies=POLICIES.get_policy_info(),
             allow_account_management=self.allow_account_management,
             account_autocreate=self.account_autocreate,
+            allow_open_expired=self.allow_open_expired,
             **constraints.EFFECTIVE_CONSTRAINTS)
         self.watchdog = Watchdog()
         self.watchdog.spawn()
@@ -638,9 +643,6 @@ class Application(object):
         timing = round(timing, 3)  # sort timings to the millisecond
         self.node_timings[node['ip']] = (timing, now + self.timing_expiry)
 
-    def _error_limit_node_key(self, node):
-        return "{ip}:{port}/{device}".format(**node)
-
     def error_limited(self, node):
         """
         Check if the node is currently error limited.
@@ -648,20 +650,11 @@ class Application(object):
         :param node: dictionary of node to check
         :returns: True if error limited, False otherwise
         """
-        now = time()
-        node_key = self._error_limit_node_key(node)
-        error_stats = self._error_limiting.get(node_key)
-
-        if error_stats is None or 'errors' not in error_stats:
-            return False
-        if 'last_error' in error_stats and error_stats['last_error'] < \
-                now - self.error_suppression_interval:
-            self._error_limiting.pop(node_key, None)
-            return False
-        limited = error_stats['errors'] > self.error_suppression_limit
+        limited = self.error_limiter.is_limited(node)
         if limited:
+            self.logger.increment('error_limiter.is_limited')
             self.logger.debug(
-                'Node error limited %(ip)s:%(port)s (%(device)s)', node)
+                'Node is error limited: %s', node_to_string(node))
         return limited
 
     def error_limit(self, node, msg):
@@ -669,24 +662,30 @@ class Application(object):
         Mark a node as error limited. This immediately pretends the
         node received enough errors to trigger error suppression. Use
         this for errors like Insufficient Storage. For other errors
-        use :func:`error_occurred`.
+        use :func:`increment`.
 
         :param node: dictionary of node to error limit
         :param msg: error message
         """
-        node_key = self._error_limit_node_key(node)
-        error_stats = self._error_limiting.setdefault(node_key, {})
-        error_stats['errors'] = self.error_suppression_limit + 1
-        error_stats['last_error'] = time()
-        self.logger.error('%(msg)s %(ip)s:%(port)s/%(device)s', {
-            'msg': msg, 'ip': node['ip'],
-            'port': node['port'], 'device': node['device']})
+        self.error_limiter.limit(node)
+        self.logger.increment('error_limiter.forced_limit')
+        self.logger.error(
+            'Node will be error limited for %.2fs: %s, error: %s',
+            self.error_limiter.suppression_interval, node_to_string(node),
+            msg)
 
-    def _incr_node_errors(self, node):
-        node_key = self._error_limit_node_key(node)
-        error_stats = self._error_limiting.setdefault(node_key, {})
-        error_stats['errors'] = error_stats.get('errors', 0) + 1
-        error_stats['last_error'] = time()
+    def _error_increment(self, node):
+        """
+        Call increment() on error limiter once, emit metrics and log if error
+        suppression will be triggered.
+
+        :param node: dictionary of node to handle errors for
+        """
+        if self.error_limiter.increment(node):
+            self.logger.increment('error_limiter.incremented_limit')
+            self.logger.error(
+                'Node will be error limited for %.2fs: %s',
+                self.error_limiter.suppression_interval, node_to_string(node))
 
     def error_occurred(self, node, msg):
         """
@@ -695,16 +694,49 @@ class Application(object):
         :param node: dictionary of node to handle errors for
         :param msg: error message
         """
-        self._incr_node_errors(node)
         if isinstance(msg, bytes):
             msg = msg.decode('utf-8')
-        self.logger.error('%(msg)s %(ip)s:%(port)s/%(device)s', {
-            'msg': msg, 'ip': node['ip'],
-            'port': node['port'], 'device': node['device']})
+        self.logger.error('%(msg)s %(node)s',
+                          {'msg': msg, 'node': node_to_string(node)})
+        self._error_increment(node)
 
-    def iter_nodes(self, ring, partition, logger, node_iter=None, policy=None):
-        return NodeIter(self, ring, partition, logger, node_iter=node_iter,
-                        policy=policy)
+    def check_response(self, node, server_type, response, method, path,
+                       body=None):
+        """
+        Check response for error status codes and update error limiters as
+        required.
+
+        :param node: a dict describing a node
+        :param server_type: the type of server from which the response was
+            received (e.g. 'Object').
+        :param response: an instance of HTTPResponse.
+        :param method: the request method.
+        :param path: the request path.
+        :param body: an optional response body. If given, up to 1024 of the
+            start of the body will be included in any log message.
+        :return True: if the response status code is less than 500, False
+            otherwise.
+        """
+        ok = False
+        if response.status == HTTP_INSUFFICIENT_STORAGE:
+            self.error_limit(node, 'ERROR Insufficient Storage')
+        elif is_server_error(response.status):
+            values = {'status': response.status,
+                      'method': method,
+                      'path': path,
+                      'type': server_type}
+            if body is None:
+                fmt = 'ERROR %(status)d Trying to %(method)s ' \
+                      '%(path)s From %(type)s Server'
+            else:
+                fmt = 'ERROR %(status)d %(body)s Trying to %(method)s ' \
+                      '%(path)s From %(type)s Server'
+                values['body'] = cap_length(body, 1024)
+            self.error_occurred(node, fmt % values)
+        else:
+            ok = True
+
+        return ok
 
     def exception_occurred(self, node, typ, additional_info,
                            **kwargs):
@@ -715,7 +747,6 @@ class Application(object):
         :param typ: server type
         :param additional_info: additional information to log
         """
-        self._incr_node_errors(node)
         if 'level' in kwargs:
             log = functools.partial(self.logger.log, kwargs.pop('level'))
             if 'exc_info' not in kwargs:
@@ -724,12 +755,12 @@ class Application(object):
             log = self.logger.exception
         if isinstance(additional_info, bytes):
             additional_info = additional_info.decode('utf-8')
-        log('ERROR with %(type)s server %(ip)s:%(port)s/%(device)s'
+        log('ERROR with %(type)s server %(node)s'
             ' re: %(info)s',
-            {'type': typ, 'ip': node['ip'],
-             'port': node['port'], 'device': node['device'],
+            {'type': typ, 'node': node_to_string(node),
              'info': additional_info},
             **kwargs)
+        self._error_increment(node)
 
     def modify_wsgi_pipeline(self, pipe):
         """
@@ -789,3 +820,12 @@ def app_factory(global_conf, **local_conf):
     app = Application(conf)
     app.check_config()
     return app
+
+
+def main():
+    conf_file, options = parse_options(test_config=True)
+    sys.exit(run_wsgi(conf_file, 'proxy-server', **options))
+
+
+if __name__ == '__main__':
+    main()
